from libfptr10 import IFptr

DTO_ERROR_DESCRIPTIONS = {
    IFptr.LIBFPTR_OK: "Ошибок нет",
    IFptr.LIBFPTR_ERROR_CONNECTION_DISABLED: "Соединение не установлено",
    IFptr.LIBFPTR_ERROR_NO_CONNECTION: "Нет связи",
    IFptr.LIBFPTR_ERROR_PORT_BUSY: "Порт занят",
    IFptr.LIBFPTR_ERROR_PORT_NOT_AVAILABLE: "Порт недоступен",
    IFptr.LIBFPTR_ERROR_INCORRECT_DATA: "Некорректные данные от устройства",
    IFptr.LIBFPTR_ERROR_INTERNAL: "Внутренняя ошибка библиотеки",
    IFptr.LIBFPTR_ERROR_UNSUPPORTED_CAST: "Неподдерживаемое преобразование типа параметра",
    IFptr.LIBFPTR_ERROR_NO_REQUIRED_PARAM: "Не найден обязательный параметр",
    IFptr.LIBFPTR_ERROR_INVALID_SETTINGS: "Некорректные настройки",
    IFptr.LIBFPTR_ERROR_NOT_CONFIGURED: "Драйвер не настроен",
    IFptr.LIBFPTR_ERROR_NOT_SUPPORTED: "Не поддерживается в данной версии",  # (драйвера или ККТ)
    IFptr.LIBFPTR_ERROR_INVALID_MODE: "Не поддерживается в данном режиме",
    IFptr.LIBFPTR_ERROR_INVALID_PARAM: "Нeкорректное значение параметра",
    IFptr.LIBFPTR_ERROR_NOT_LOADED: "Не удалось загрузить библиотеку",
    IFptr.LIBFPTR_ERROR_UNKNOWN: "Неизвестная ошибка",
    IFptr.LIBFPTR_ERROR_INVALID_SUM: "Неверная цена (сумма)",
    IFptr.LIBFPTR_ERROR_INVALID_QUANTITY: "Неверное количество",
    IFptr.LIBFPTR_ERROR_CASH_COUNTER_OVERFLOW: "Переполнение счетчика наличности",
    IFptr.LIBFPTR_ERROR_LAST_OPERATION_STORNO_DENIED: "Невозможно сторно последней операции",
    IFptr.LIBFPTR_ERROR_STORNO_BY_CODE_DENIED: "Сторно по коду невозможно",
    IFptr.LIBFPTR_ERROR_LAST_OPERATION_NOT_REPEATABLE: "Невозможен повтор последней операции",
    IFptr.LIBFPTR_ERROR_DISCOUNT_NOT_REPEATABLE: "Повторная скидка на операцию невозможна",
    IFptr.LIBFPTR_ERROR_DISCOUNT_DENIED: "Невозможно начислить скидку/надбавку",
    IFptr.LIBFPTR_ERROR_INVALID_COMMODITY_CODE: "Неверный код товара",
    IFptr.LIBFPTR_ERROR_INVALID_COMMODITY_BARCODE: "Неверный штрихкод товара",
    IFptr.LIBFPTR_ERROR_INVALID_COMMAND_FORMAT: "Неверный формат команды",
    IFptr.LIBFPTR_ERROR_INVALID_COMMAND_LENGTH: "Неверная длина",
    IFptr.LIBFPTR_ERROR_BLOCKED_IN_DATE_INPUT_MODE: "ККТ заблокирована в режиме ввода даты",
    IFptr.LIBFPTR_ERROR_NEED_DATE_ACCEPT: "Требуется подтверждение ввода даты",
    IFptr.LIBFPTR_ERROR_NO_MORE_DATA: "Нет больше данных",
    IFptr.LIBFPTR_ERROR_NO_ACCEPT_OR_CANCEL: "Нет подтверждения или отмены продажи",
    IFptr.LIBFPTR_ERROR_BLOCKED_BY_REPORT_INTERRUPTION: "Отчет о закрытии смены прерван",
    IFptr.LIBFPTR_ERROR_DISABLE_CASH_CONTROL_DENIED: "Отключение контроля наличности невозможно "
    "(не настроены необходимые типы оплаты)",
    IFptr.LIBFPTR_ERROR_MODE_BLOCKED: "Вход в режим заблокирован",
    IFptr.LIBFPTR_ERROR_CHECK_DATE_TIME: "Проверьте дату и время",
    IFptr.LIBFPTR_ERROR_DATE_TIME_LESS_THAN_FS: "Переданные дата/время меньше "
    "даты/времени последнего фискального документа",
    IFptr.LIBFPTR_ERROR_CLOSE_ARCHIVE_DENIED: "Невозможно закрыть архив",
    IFptr.LIBFPTR_ERROR_COMMODITY_NOT_FOUND: "Товар не найден",
    IFptr.LIBFPTR_ERROR_WEIGHT_BARCODE_WITH_INVALID_QUANTITY: "Весовой штрихкод с количеством <> 1.000",
    IFptr.LIBFPTR_ERROR_RECEIPT_BUFFER_OVERFLOW: "Переполнение буфера чека",
    IFptr.LIBFPTR_ERROR_QUANTITY_TOO_FEW: "Недостаточное количество товара",
    IFptr.LIBFPTR_ERROR_STORNO_TOO_MUCH: "Сторнируемое количество больше проданного",
    IFptr.LIBFPTR_ERROR_BLOCKED_COMMODITY_NOT_FOUND: "Товар не найден",
    IFptr.LIBFPTR_ERROR_NO_PAPER: "Нет бумаги",
    IFptr.LIBFPTR_ERROR_COVER_OPENED: "Открыта крышка",
    IFptr.LIBFPTR_ERROR_PRINTER_FAULT: "Нет связи с принтером чеков",
    IFptr.LIBFPTR_ERROR_MECHANICAL_FAULT: "Механическая ошибка печатающего устройства",
    IFptr.LIBFPTR_ERROR_INVALID_RECEIPT_TYPE: "Неверный тип чека",
    IFptr.LIBFPTR_ERROR_INVALID_UNIT_TYPE: "Недопустимое целевое устройство",
    IFptr.LIBFPTR_ERROR_NO_MEMORY: "Нет места в массиве картинок/штрихкодов",
    IFptr.LIBFPTR_ERROR_PICTURE_NOT_FOUND: "Неверный номер картинки/штрихкода (картинка/штрихкод отсутствует)",
    IFptr.LIBFPTR_ERROR_NONCACH_PAYMENTS_TOO_MUCH: "Сумма не наличных платежей превышает сумму чека",
    IFptr.LIBFPTR_ERROR_RETURN_DENIED: "Накопление меньше суммы возврата или аннулирования",
    IFptr.LIBFPTR_ERROR_PAYMENTS_OVERFLOW: "Переполнение суммы платежей",
    IFptr.LIBFPTR_ERROR_BUSY: "Предыдущая операция незавершена",
    IFptr.LIBFPTR_ERROR_GSM: "Ошибка GSM-модуля",
    IFptr.LIBFPTR_ERROR_INVALID_DISCOUNT: "Неверная величина скидки / надбавки",
    IFptr.LIBFPTR_ERROR_OPERATION_AFTER_DISCOUNT_DENIED: "Операция после скидки / надбавки невозможна",
    IFptr.LIBFPTR_ERROR_INVALID_DEPARTMENT: "Неверная секция",
    IFptr.LIBFPTR_ERROR_INVALID_PAYMENT_TYPE: "Неверный вид оплаты",
    IFptr.LIBFPTR_ERROR_MULTIPLICATION_OVERFLOW: "Переполнение при умножении",
    IFptr.LIBFPTR_ERROR_DENIED_BY_SETTINGS: "Операция запрещена в таблице настроек",
    IFptr.LIBFPTR_ERROR_TOTAL_OVERFLOW: "Переполнение итога чека",
    IFptr.LIBFPTR_ERROR_DENIED_IN_ANNULATION_RECEIPT: "Открыт чек аннулирования – операция невозможна",
    IFptr.LIBFPTR_ERROR_JOURNAL_OVERFLOW: "Переполнение буфера ЭЖ",
    IFptr.LIBFPTR_ERROR_NOT_FULLY_PAID: "Чек оплачен не полностью",
    IFptr.LIBFPTR_ERROR_DENIED_IN_RETURN_RECEIPT: "Открыт чек возврата – операция невозможна",
    IFptr.LIBFPTR_ERROR_SHIFT_EXPIRED: "Смена превысила 24 часа",
    IFptr.LIBFPTR_ERROR_DENIED_IN_SELL_RECEIPT: "Открыт чек продажи – операция невозможна",
    IFptr.LIBFPTR_ERROR_FISCAL_MEMORY_OVERFLOW: "Переполнение ФП",
    IFptr.LIBFPTR_ERROR_INVALID_PASSWORD: "Неверный пароль",
    IFptr.LIBFPTR_ERROR_JOURNAL_BUSY: "Идет обработка ЭЖ",
    IFptr.LIBFPTR_ERROR_DENIED_IN_CLOSED_SHIFT: "Смена закрыта - операция невозможна",
    IFptr.LIBFPTR_ERROR_INVALID_TABLE_NUMBER: "Неверный номер таблицы",
    IFptr.LIBFPTR_ERROR_INVALID_ROW_NUMBER: "Неверный номер ряда",
    IFptr.LIBFPTR_ERROR_INVALID_FIELD_NUMBER: "Неверный номер поля",
    IFptr.LIBFPTR_ERROR_INVALID_DATE_TIME: "Неверная дата и/или время",
    IFptr.LIBFPTR_ERROR_INVALID_STORNO_SUM: "Неверная сумма сторно",
    IFptr.LIBFPTR_ERROR_CHANGE_CALCULATION: "Подсчет суммы сдачи невозможен",
    IFptr.LIBFPTR_ERROR_NO_CASH: "В ККТ нет денег для выплаты",
    IFptr.LIBFPTR_ERROR_DENIED_IN_CLOSED_RECEIPT: "Чек закрыт – операция невозможна",
    IFptr.LIBFPTR_ERROR_DENIED_IN_OPENED_RECEIPT: "Чек открыт – операция невозможна",
    IFptr.LIBFPTR_ERROR_DENIED_IN_OPENED_SHIFT: "Смена открыта, операция невозможна",
    IFptr.LIBFPTR_ERROR_SERIAL_NUMBER_ALREADY_ENTERED: "Серийный номер / MAC-адрес уже задан",
    IFptr.LIBFPTR_ERROR_TOO_MUCH_REREGISTRATIONS: "Исчерпан лимит перерегистраций",
    IFptr.LIBFPTR_ERROR_INVALID_SHIFT_NUMBER: "Неверный номер смены",
    IFptr.LIBFPTR_ERROR_INVALID_SERIAL_NUMBER: "Недопустимый серийный номер ККТ",
    IFptr.LIBFPTR_ERROR_INVALID_RNM_VATIN: "Недопустимый РНМ и/или ИНН",
    IFptr.LIBFPTR_ERROR_FISCAL_PRINTER_NOT_ACTIVATED: "ККТ не зарегистрирована",
    IFptr.LIBFPTR_ERROR_SERIAL_NUMBER_NOT_ENTERED: "Не задан серийный номер",
    IFptr.LIBFPTR_ERROR_NO_MORE_REPORTS: "Нет отчетов",
    IFptr.LIBFPTR_ERROR_MODE_NOT_ACTIVATED: "Режим не активизирован",
    IFptr.LIBFPTR_ERROR_RECORD_NOT_FOUND_IN_JOURNAL: "Данные документа отсутствуют",
    IFptr.LIBFPTR_ERROR_INVALID_LICENSE: "Некорректный код защиты / лицензия или номер",
    IFptr.LIBFPTR_ERROR_NEED_FULL_RESET: "Требуется выполнение общего гашения",
    IFptr.LIBFPTR_ERROR_DENIED_BY_LICENSE: "Команда не разрешена введенными кодами защиты / лицензиями ККТ",
    IFptr.LIBFPTR_ERROR_DISCOUNT_CANCELLATION_DENIED: "Невозможна отмена скидки/надбавки",
    IFptr.LIBFPTR_ERROR_CLOSE_RECEIPT_DENIED: "Невозможно закрыть чек данным типом оплаты",
    IFptr.LIBFPTR_ERROR_INVALID_ROUTE_NUMBER: "Неверный номер маршрута",
    IFptr.LIBFPTR_ERROR_INVALID_START_ZONE_NUMBER: "Неверный номер начальной зоны",
    IFptr.LIBFPTR_ERROR_INVALID_END_ZONE_NUMBER: "Неверный номер конечной зоны",
    IFptr.LIBFPTR_ERROR_INVALID_RATE_TYPE: "Неверный тип тарифа",
    IFptr.LIBFPTR_ERROR_INVALID_RATE: "Неверный тариф",
    IFptr.LIBFPTR_ERROR_FISCAL_MODULE_EXCHANGE: "Ошибка обмена с фискальным модулем",
    IFptr.LIBFPTR_ERROR_NEED_TECHNICAL_SUPPORT: "Необходимо провести профилактические работы",
    IFptr.LIBFPTR_ERROR_SHIFT_NUMBERS_DID_NOT_MATCH: "Неверные номера смен в ККТ и ФН",
    IFptr.LIBFPTR_ERROR_DEVICE_NOT_FOUND: "Нет устройства, обрабатывающего данную команду",
    IFptr.LIBFPTR_ERROR_EXTERNAL_DEVICE_CONNECTION: "Нет связи с внешним устройством",
    IFptr.LIBFPTR_ERROR_DISPENSER_INVALID_STATE: "Ошибочное состояние ТРК",
    IFptr.LIBFPTR_ERROR_INVALID_POSITIONS_COUNT: "Недопустимое кол-во позиций в чеке",
    IFptr.LIBFPTR_ERROR_DISPENSER_INVALID_NUMBER: "Ошибочный номер ТРК",
    IFptr.LIBFPTR_ERROR_INVALID_DIVIDER: "Неверный делитель",
    IFptr.LIBFPTR_ERROR_FN_ACTIVATION_DENIED: "Активация данного ФН в составе данной ККТ невозможна",
    IFptr.LIBFPTR_ERROR_PRINTER_OVERHEAT: "Перегрев головки принтера",
    IFptr.LIBFPTR_ERROR_FN_EXCHANGE: "Ошибка обмена с ФН на уровне интерфейса I2C",
    IFptr.LIBFPTR_ERROR_FN_INVALID_FORMAT: "Ошибка формата передачи ФН",
    IFptr.LIBFPTR_ERROR_FN_INVALID_STATE: "Неверное состояние ФН",
    IFptr.LIBFPTR_ERROR_FN_FAULT: "Неисправимая ошибка ФН",
    IFptr.LIBFPTR_ERROR_FN_CRYPTO_FAULT: "Ошибка КС ФН",
    IFptr.LIBFPTR_ERROR_FN_EXPIRED: "Закончен срок эксплуатации ФН",
    IFptr.LIBFPTR_ERROR_FN_OVERFLOW: "Архив ФН переполнен",
    IFptr.LIBFPTR_ERROR_FN_INVALID_DATE_TIME: "В ФН переданы неверная дата или время",
    IFptr.LIBFPTR_ERROR_FN_NO_MORE_DATA: "В ФН нет запрошенных данных",
    IFptr.LIBFPTR_ERROR_FN_TOTAL_OVERFLOW: "Переполнение ФН (итог чека)",
    IFptr.LIBFPTR_ERROR_BUFFER_OVERFLOW: "Буфер переполнен",
    IFptr.LIBFPTR_ERROR_PRINT_SECOND_COPY_DENIED: "Невозможно напечатать вторую фискальную копию",
    IFptr.LIBFPTR_ERROR_NEED_RESET_JOURNAL: "Требуется гашение ЭЖ",
    IFptr.LIBFPTR_ERROR_TAX_SUM_TOO_MUCH: "Некорректная сумма налога",
    IFptr.LIBFPTR_ERROR_TAX_ON_LAST_OPERATION_DENIED: "Начисление налога на последнюю операцию невозможно",
    IFptr.LIBFPTR_ERROR_INVALID_FN_NUMBER: "Неверный номер ФН",
    IFptr.LIBFPTR_ERROR_TAX_CANCEL_DENIED: "Сумма сторно налога больше суммы зарегистрированного налога данного типа",
    IFptr.LIBFPTR_ERROR_LOW_BATTERY: "Операция невозможна, недостаточно питания",
    IFptr.LIBFPTR_ERROR_FN_INVALID_COMMAND: "Некорректное значение параметров команды ФН",
    IFptr.LIBFPTR_ERROR_FN_COMMAND_OVERFLOW: "Превышение размеров TLV данных ФН",
    IFptr.LIBFPTR_ERROR_FN_NO_TRANSPORT_CONNECTION: "Нет транспортного соединения ФН",
    IFptr.LIBFPTR_ERROR_FN_CRYPTO_HAS_EXPIRED: "Исчерпан ресурс КС ФН",
    IFptr.LIBFPTR_ERROR_FN_RESOURCE_HAS_EXPIRED: "Ресурс хранения ФД исчерпан",
    IFptr.LIBFPTR_ERROR_INVALID_MESSAGE_FROM_OFD: "Сообщение от ОФД не может быть принято ФН",
    IFptr.LIBFPTR_ERROR_FN_HAS_NOT_SEND_DOCUMENTS: "В ФН есть неотправленные ФД",
    IFptr.LIBFPTR_ERROR_FN_TIMEOUT: "Исчерпан ресурс ожидания передачи сообщения в ФН",
    IFptr.LIBFPTR_ERROR_FN_SHIFT_EXPIRED: "Продолжительность смены ФН более 24 часов",
    IFptr.LIBFPTR_ERROR_FN_INVALID_TIME_DIFFERENCE: "Неверная разница во времени между двумя операциями ФН",
    IFptr.LIBFPTR_ERROR_INVALID_TAXATION_TYPE: "Некорректная СНО",
    IFptr.LIBFPTR_ERROR_INVALID_TAX_TYPE: "Недопустимый номер ставки налога",
    IFptr.LIBFPTR_ERROR_INVALID_COMMODITY_PAYMENT_TYPE: "Недопустимый тип оплаты товара",
    IFptr.LIBFPTR_ERROR_INVALID_COMMODITY_CODE_TYPE: "Недопустимый тип кода товара",
    IFptr.LIBFPTR_ERROR_EXCISABLE_COMMODITY_DENIED: "Недопустима регистрация подакцизного товара",
    IFptr.LIBFPTR_ERROR_FISCAL_PROPERTY_WRITE: "Ошибка программирования реквизита",
    IFptr.LIBFPTR_ERROR_INVALID_COUNTER_TYPE: "Неверный тип счетчика",
    IFptr.LIBFPTR_ERROR_CUTTER_FAULT: "Ошибка отрезчика",
    IFptr.LIBFPTR_ERROR_REPORT_INTERRUPTED: "Снятие отчета прервалось",
    IFptr.LIBFPTR_ERROR_INVALID_LEFT_MARGIN: "Недопустимое значение отступа слева",
    IFptr.LIBFPTR_ERROR_INVALID_ALIGNMENT: "Недопустимое значение выравнивания",
    IFptr.LIBFPTR_ERROR_INVALID_TAX_MODE: "Недопустимое значение режима работы с налогом",
    IFptr.LIBFPTR_ERROR_FILE_NOT_FOUND: "Файл не найден или неверный формат",
    IFptr.LIBFPTR_ERROR_PICTURE_TOO_BIG: "Размер картинки слишком большой",
    IFptr.LIBFPTR_ERROR_INVALID_BARCODE_PARAMS: "Не удалось сформировать штрихкод",
    IFptr.LIBFPTR_ERROR_FISCAL_PROPERTY_DENIED: "Неразрешенные реквизиты",
    IFptr.LIBFPTR_ERROR_FN_INTERFACE: "Ошибка интерфейса ФН",
    IFptr.LIBFPTR_ERROR_DATA_DUPLICATE: "Дублирование данных",
    IFptr.LIBFPTR_ERROR_NO_REQUIRED_FISCAL_PROPERTY: "Не указаны обязательные реквизиты",
    IFptr.LIBFPTR_ERROR_FN_READ_DOCUMENT: "Ошибка чтения документа из ФН",
    IFptr.LIBFPTR_ERROR_FLOAT_OVERFLOW: "Переполнение чисел с плавающей точкой",
    IFptr.LIBFPTR_ERROR_INVALID_SETTING_VALUE: "Неверное значение параметра ККТ",
    IFptr.LIBFPTR_ERROR_HARD_FAULT: "Внутренняя ошибка ККТ",
    IFptr.LIBFPTR_ERROR_FN_NOT_FOUND: "ФН не найден",
    IFptr.LIBFPTR_ERROR_INVALID_AGENT_FISCAL_PROPERTY: "Невозможно записать реквизит агента",
    IFptr.LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1002_1056: "Недопустимое сочетание реквизитов 1002 и 1056",
    IFptr.LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1002_1017: "Недопустимое сочетание реквизитов 1002 и 1017",
    IFptr.LIBFPTR_ERROR_SCRIPT: "Ошибка скриптового движка ККТ",
    IFptr.LIBFPTR_ERROR_INVALID_USER_MEMORY_INDEX: "Неверный номер пользовательской ячейки памяти",
    IFptr.LIBFPTR_ERROR_NO_ACTIVE_OPERATOR: "Кассир не зарегистрирован",
    IFptr.LIBFPTR_ERROR_REGISTRATION_REPORT_INTERRUPTED: "Отчет о регистрации ККТ прерван",
    IFptr.LIBFPTR_ERROR_CLOSE_FN_REPORT_INTERRUPTED: "Отчет о закрытии ФН прерван",
    IFptr.LIBFPTR_ERROR_OPEN_SHIFT_REPORT_INTERRUPTED: "Отчет об открытии смены прерван",
    IFptr.LIBFPTR_ERROR_OFD_EXCHANGE_REPORT_INTERRUPTED: "Отчет о состоянии расчетов прерван",
    IFptr.LIBFPTR_ERROR_CLOSE_RECEIPT_INTERRUPTED: "Закрытие чека прервано",
    IFptr.LIBFPTR_ERROR_FN_QUERY_INTERRUPTED: "Получение документа из ФН прервано",
    IFptr.LIBFPTR_ERROR_RTC_FAULT: "Сбой часов",
    IFptr.LIBFPTR_ERROR_MEMORY_FAULT: "Сбой памяти",
    IFptr.LIBFPTR_ERROR_CHIP_FAULT: "Сбой микросхемы",
    IFptr.LIBFPTR_ERROR_TEMPLATES_CORRUPTED: "Ошибка шаблонов документов",
    IFptr.LIBFPTR_ERROR_INVALID_MAC_ADDRESS: "Недопустимое значение MAC-адреса",
    IFptr.LIBFPTR_ERROR_INVALID_SCRIPT_NUMBER: "Неверный тип (номер) шаблона",
    IFptr.LIBFPTR_ERROR_SCRIPTS_FAULT: "Загруженные шаблоны повреждены или отсутствуют",
    IFptr.LIBFPTR_ERROR_INVALID_SCRIPTS_VERSION: "Несовместимая версия загруженных шаблонов",
    IFptr.LIBFPTR_ERROR_INVALID_CLICHE_FORMAT: "Ошибка в формате клише",
    IFptr.LIBFPTR_ERROR_WAIT_FOR_REBOOT: "Требуется перезагрузка ККТ",
    IFptr.LIBFPTR_ERROR_NO_LICENSE: "Подходящие лицензии не найдены",
    IFptr.LIBFPTR_ERROR_INVALID_FFD_VERSION: "Неверная версия ФФД",
    IFptr.LIBFPTR_ERROR_CHANGE_SETTING_DENIED: "Параметр доступен только для чтения",
    IFptr.LIBFPTR_ERROR_INVALID_NOMENCLATURE_TYPE: "Неверный тип кода товара",
    IFptr.LIBFPTR_ERROR_INVALID_GTIN: "Неверное значение GTIN",
    IFptr.LIBFPTR_ERROR_NEGATIVE_MATH_RESULT: "Отрицательный результат математической операции",
    IFptr.LIBFPTR_ERROR_FISCAL_PROPERTIES_COMBINATION: "Недопустимое сочетание реквизитов",
    IFptr.LIBFPTR_ERROR_OPERATOR_LOGIN: "Не удалось зарегистрировать кассира",
    IFptr.LIBFPTR_ERROR_INVALID_INTERNET_CHANNEL: "Данный канал Интернет отсутствует в ККТ",
    IFptr.LIBFPTR_ERROR_DATETIME_NOT_SYNCRONIZED: "Дата и время не синхронизированы",
    IFptr.LIBFPTR_ERROR_JOURNAL: "Ошибка электронного журнала",
    IFptr.LIBFPTR_ERROR_DENIED_IN_OPENED_DOC: "Документ открыт - операция невозможна",
    IFptr.LIBFPTR_ERROR_DENIED_IN_CLOSED_DOC: "Документ закрыт - операция невозможна",
    IFptr.LIBFPTR_ERROR_LICENSE_MEMORY_OVERFLOW: "Нет места для сохранения лицензий",
    IFptr.LIBFPTR_ERROR_NEED_CANCEL_DOCUMENT: "Произошла критичная ошибка, документ необходимо отменить",
    IFptr.LIBFPTR_ERROR_REGISTERS_NOT_INITIALIZED: "Регистры ККТ еще не инициализированы",
    IFptr.LIBFPTR_ERROR_TOTAL_REQUIRED: "Требуется регистрация итога",
    IFptr.LIBFPTR_ERROR_SETTINGS_FAULT: "Сбой таблицы настроек",
    IFptr.LIBFPTR_ERROR_COUNTERS_FAULT: "Сбой счетчиков и регистров ККТ",
    IFptr.LIBFPTR_ERROR_USER_MEMORY_FAULT: "Сбой пользовательской памяти",
    IFptr.LIBFPTR_ERROR_SERVICE_COUNTERS_FAULT: "Сбой сервисных регистров",
    IFptr.LIBFPTR_ERROR_ATTRIBUTES_FAULT: "Сбой реквизитов ККТ",
    IFptr.LIBFPTR_ERROR_ALREADY_IN_UPDATE_MODE: "ККТ уже в режиме обновления конфигурации",
    IFptr.LIBFPTR_ERROR_INVALID_FIRMWARE: "Конфигурация не прошла проверку",
    IFptr.LIBFPTR_ERROR_INVALID_CHANNEL: "Аппаратный канал отсутствует, выключен или ещё не проинициализирован",
    IFptr.LIBFPTR_ERROR_INTERFACE_DOWN: "Сетевой интерфейс не подключен, или на нём не получен IP-адрес",
    IFptr.LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1212_1030: "Недопустимое сочетание реквизитов 1212 и 1030",
    IFptr.LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1214: "Некорректный признак способа расчета",
    IFptr.LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1212: "Некорректный признак предмета расчета",
    IFptr.LIBFPTR_ERROR_SYNC_TIME: "Ошибка синхронизации времени",
    IFptr.LIBFPTR_ERROR_VAT18_VAT20_IN_RECEIPT: "В одном чеке одновременно не может быть позиций с НДС 18% (18/118) и "
    "НДС 20% (20/120)",
    IFptr.LIBFPTR_ERROR_PICTURE_NOT_CLOSED: "Картинка не закрыта",
    IFptr.LIBFPTR_ERROR_INTERFACE_BUSY: "Сетевой интерфейс занят",
    IFptr.LIBFPTR_ERROR_INVALID_PICTURE_NUMBER: "Неверный номер картинки",
    IFptr.LIBFPTR_ERROR_INVALID_CONTAINER: "Ошибка проверки контейнера",
    IFptr.LIBFPTR_ERROR_ARCHIVE_CLOSED: "Архив ФН закрыт",
    IFptr.LIBFPTR_ERROR_NEED_REGISTRATION: "Нужно выполнить регистрацию / перерегистрацию",
    IFptr.LIBFPTR_ERROR_DENIED_DURING_UPDATE: "Операция невозможна, идет обновление ПО ККТ",
    IFptr.LIBFPTR_ERROR_INVALID_TOTAL: "Неверный итог чека",
    IFptr.LIBFPTR_ERROR_MARKING_CODE_CONFLICT: "Запрещена одновременная передача КМ и реквизита 1162",
    IFptr.LIBFPTR_ERROR_INVALID_RECORDS_ID: "Набор записей по заданному идентификатору не найден",
    IFptr.LIBFPTR_ERROR_INVALID_SIGNATURE: "Ошибка цифровой подписи",
    IFptr.LIBFPTR_ERROR_INVALID_EXCISE_SUM: "Некорректная сумма акциза",
    IFptr.LIBFPTR_ERROR_NO_DOCUMENTS_FOUND_IN_JOURNAL: "Заданный диапазон документов не найден в БД документов",
    IFptr.LIBFPTR_ERROR_INVALID_SCRIPT_TYPE: "Неподдерживаемый тип скрипта",
    IFptr.LIBFPTR_ERROR_INVALID_SCRIPT_NAME: "Некорректный идентификатор скрипта",
    IFptr.LIBFPTR_ERROR_INVALID_POSITIONS_COUNT_WITH_1162: "Кол-во позиций с реквизитом 1162/1163 в автономном режиме "
    "превысило разрешенный лимит",
    IFptr.LIBFPTR_ERROR_INVALID_UC_COUNTER: "Универсальный счетчик с заданными параметрами недоступен",
    IFptr.LIBFPTR_ERROR_INVALID_UC_TAG: "Неподдерживаемый тег для универсальных счетчиков",
    IFptr.LIBFPTR_ERROR_INVALID_UC_IDX: "Некорректный индекс для универсальных счетчиков",
    IFptr.LIBFPTR_ERROR_INVALID_UC_SIZE: "Неверный размер универсального счетчика",
    IFptr.LIBFPTR_ERROR_INVALID_UC_CONFIG: "Неверная конфигурация универсальных счетчиков",
    IFptr.LIBFPTR_ERROR_CONNECTION_LOST: "Соединение с ККТ потеряно",
    IFptr.LIBFPTR_ERROR_UNIVERSAL_COUNTERS_FAULT: "Ошибка универсальных счетчиков",
    IFptr.LIBFPTR_ERROR_INVALID_TAX_SUM: "Некорректная сумма налога",
    IFptr.LIBFPTR_ERROR_INVALID_MARKING_CODE_TYPE: "Некорректное значение типа кода маркировки",
    IFptr.LIBFPTR_ERROR_LICENSE_HARD_FAULT: "Аппаратная ошибка при сохранении лицензии",
    IFptr.LIBFPTR_ERROR_LICENSE_INVALID_SIGN: "Подпись лицензии некорректна",
    IFptr.LIBFPTR_ERROR_LICENSE_INVALID_SERIAL: "Лицензия не подходит для данной ККТ",
    IFptr.LIBFPTR_ERROR_LICENSE_INVALID_TIME: "Срок действия лицензии истёк",
    IFptr.LIBFPTR_ERROR_DOCUMENT_CANCELED: "Документ был отменен",
    IFptr.LIBFPTR_ERROR_INVALID_SCRIPT_PARAMS: "Некорректные параметры скрипта",
    IFptr.LIBFPTR_ERROR_CLICHE_TOO_LONG: "Длина клише превышает максимальное значение",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE_FAULT: "Ошибка таблицы товаров",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE: "Общая ошибка таблицы товаров",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE_INVALID_TAG: "Некорректный тег для таблицы товаров",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE_INVALID_TAG_SIZE: "Некорректный размер тега для таблицы товаров",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE_NO_TAG_DATA: "Нет данных по тегу в таблице товаров",
    IFptr.LIBFPTR_ERROR_COMMODITIES_TABLE_NO_FREE_MEMORY: "Нет места в динамической области памяти таблицы товаров",
    IFptr.LIBFPTR_ERROR_INVALID_CACHE: "Ошибка чтения/записи данных кеша",
    IFptr.LIBFPTR_ERROR_SCHEDULER_NOT_READY: "Функции планировщика заданий не доступны",
    IFptr.LIBFPTR_ERROR_SCHEDULER_INVALID_TASK: "Неизвестный тип задания планировщика",
    IFptr.LIBFPTR_ERROR_MINIPOS_NO_POSITION_PAYMENT: "Отсутствует позиция оплаты",
    IFptr.LIBFPTR_ERROR_MINIPOS_COMMAND_TIME_OUT: "Таймаут выполнения команды истек",
    IFptr.LIBFPTR_ERROR_MINIPOS_MODE_FR_DISABLED: "Режим ФР выключен",
    IFptr.LIBFPTR_ERROR_ENTRY_NOT_FOUND_IN_OTP: "Не найдена запись в OTP",
    IFptr.LIBFPTR_ERROR_EXCISABLE_COMMODITY_WITHOUT_EXCISE: "Подакцизный товар без акциза зарегистрирован в чеке",
    IFptr.LIBFPTR_ERROR_BARCODE_TYPE_NOT_SUPPORTED: "Данный тип штрихкода не поддерживается",
    IFptr.LIBFPTR_ERROR_OVERLAY_DATA_OVERFLOW: "Размер данных штрихкода и текста превышает допустимый",
    IFptr.LIBFPTR_ERROR_INVALID_MODULE_ADDRESS: "Ошибка чтения адреса модуля и сегмента",
    IFptr.LIBFPTR_ERROR_ECR_MODEL_NOT_SUPPORTED: "Данная модель ККТ не поддерживается",
    IFptr.LIBFPTR_ERROR_MARKING_CODE_VALIDATION_IN_PROGRESS: "Процедура проверки КМ уже запущена",
    IFptr.LIBFPTR_ERROR_NO_CONNECTION_WITH_SERVER: "Ошибка соединения с сервером",
    IFptr.LIBFPTR_ERROR_MARKING_CODE_VALIDATION_CANCELED: "Процедура проверки КМ прервана",
    IFptr.LIBFPTR_ERROR_INVALID_MARKING_CODE_STATUS: "Некорректное значение статуса КМ",
    IFptr.LIBFPTR_ERROR_INVALID_GS1: "Неверный код GS1",
    IFptr.LIBFPTR_ERROR_MARKING_WORK_DENIED: "Запрещена работа с маркированным товарами",
    IFptr.LIBFPTR_ERROR_MARKING_WORK_TEMPORARY_BLOCKED: "Работа с маркированными товарами временно заблокирована",
    IFptr.LIBFPTR_ERROR_MARKS_OVERFLOW: "Переполнена таблица хранения КМ",
    IFptr.LIBFPTR_ERROR_INVALID_MARKING_CODE: "Некорректный код маркировки",
    IFptr.LIBFPTR_ERROR_INVALID_STATE: "Неверное состояние",
    IFptr.LIBFPTR_ERROR_OFD_EXCHANGE: "Ошибка обмена с сервером ОФД или ИСМ",
    IFptr.LIBFPTR_ERROR_INVALID_MEASUREMENT_UNIT: "Некорректное значение единиц измерения",
    IFptr.LIBFPTR_ERROR_OPERATION_DENIED_IN_CURRENT_FFD: "Операция не разрешена в данной версии ФФД",
    IFptr.LIBFPTR_ERROR_MARKING_OPERATION_DENIED: "Операция не разрешена, "
    "при регистрации не был установлен признак ТМТ",
    IFptr.LIBFPTR_ERROR_NO_DATA_TO_SEND: "Нет данных для отправки",
    IFptr.LIBFPTR_ERROR_NO_MARKED_POSITION: "Нет маркированных позиций в чеке",
    IFptr.LIBFPTR_ERROR_HAS_NOT_SEND_NOTICES: "Имеются неотправленные уведомления",
    IFptr.LIBFPTR_ERROR_UPDATE_KEYS_REQUIRED: "Требуется повторное проведение процедуры обновления ключей",
    IFptr.LIBFPTR_ERROR_UPDATE_KEYS_SERVICE: "Ошибка сервиса обновления ключей проверки КМ",
    IFptr.LIBFPTR_ERROR_MARK_NOT_CHECKED: "КМ не проверен в ФН",
    IFptr.LIBFPTR_ERROR_MARK_CHECK_TIMEOUT_EXPIRED: "Истёк таймаут проверки КМ",
    IFptr.LIBFPTR_ERROR_NO_MARKING_CODE_IN_TABLE: "Данный КМ отсутствует в таблице",
    IFptr.LIBFPTR_ERROR_CHEKING_MARK_IN_PROGRESS: "Выполняется проверка КМ",
    IFptr.LIBFPTR_ERROR_INVALID_SERVER_ADDRESS: "Адрес сервера ИСМ не задан",
    IFptr.LIBFPTR_ERROR_UPDATE_KEYS_TIMEOUT: "Истёк таймаут обновления ключей проверки",
    IFptr.LIBFPTR_ERROR_PROPERTY_FOR_MARKING_POSITION_ONLY: "Данный реквизит разрешён только для маркированной позиции",
    IFptr.LIBFPTR_ERROR_RECEIPT_PARSE_ERROR: "Ошибка парсинга запроса",
    IFptr.LIBFPTR_ERROR_INTERRUPTED_BY_PREVIOUS_ERRORS: "Выполнение прервано из-за предыдущих ошибок",
    IFptr.LIBFPTR_ERROR_DRIVER_SCRIPT_ERROR: "Ошибка скрипта драйвера",
    IFptr.LIBFPTR_ERROR_VALIDATE_FUNC_NOT_FOUND: "Функция проверки задания не найдена",
    IFptr.LIBFPTR_ERROR_RCP_SERVER_BUSY: "Устройство на сервере удалённого подключения занято другим клиентом",
    IFptr.LIBFPTR_ERROR_RCP_SERVER_VERSION: "Некорректная версия протокола обмена с сервером удалённого подключения",
    IFptr.LIBFPTR_ERROR_RCP_SERVER_EXCHANGE: "Ошибка обмена с сервером удалённого подключения",
}


def get_dto_error_code_description(error_code):
    """Запрос описания ошибки ДТО"""
    if error_code in DTO_ERROR_DESCRIPTIONS:
        return DTO_ERROR_DESCRIPTIONS[error_code]
    else:
        return "НЕИЗВЕСТНАЯ ОШИБКА ДРАЙВЕРА! НЕОБХОДИМО АКТУАЛИЗИРОВАТЬ СЛОВАРЬ const/dto_error_descriptions.py"



import json
import re
import sys
import time
from datetime import datetime

DOSSymbols = (
    ' !"#$%&`()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'
    " АБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдежзийклмноп                                              "
    "  рстуфхцчшщъыьэюяЁё          №   "
)

KKTSymbols = (
    ' !"#№%&`()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'
    " АБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдежзийклмноп                                              "
    "  рстуфхцчшщъыьэюяЁё€         $   "
)

InKKTSymbols = (
    'АБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ !"#№%&`()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^'
    "_`abcdefghijklmnopqrstuvwxyz{|}~ абвгдежзийклмнопрстуфхцчшщъыьэюя$€-                            "
    "                                                 Ёё            \t "
)

OutKKTSymbols = (
    '         \t                       !"#№%&`()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^'
    "_`abcdefghijklmnopqrstuvwxyz{|}~ АБВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдежзийклмноп              "
    "                                  рстуфхцчшщъыьэюяЁё€       - $   "
)


# noinspection PyPep8Naming
def DictToStr(Dictionary):
    return json.dumps(Dictionary, ensure_ascii=False, encoding="utf-8")


def ByteToDOS(byte):
    return DOSSymbols[byte - 0x20]


def DOSToByte(char):
    for i in range(256):
        if DOSSymbols[i] == char:
            return i + 0x20
    return 0x20


def ByteToKKTSymbols(char):
    return KKTSymbols[char]


def ByteToInKKTSymbols(char):
    return InKKTSymbols[char]


def KKTSymbToByte(char):
    for i in range(256):
        if KKTSymbols[i] == char:
            return i + 0x20
    return 0x20


def ArrayToStr(array):
    result = ""
    for code in array:
        result += chr(code)
    return result


def StringToBytes(string):
    result = ""
    for CHR in string:
        result = result + HexToStr(DOSToByte(CHR))
    return result


def StringToBytesKKT(string):
    result = ""
    for CHR in string:
        result = result + HexToStr(KKTSymbToByte(CHR))
    return result


def BytesKKTInToString(array):
    result = ""
    for Byte in array:
        result += InKKTSymbols[Byte]
    return result


def ArrayToHexString(array):
    result = ""
    space = False
    for Byte in array:
        result += HexToStr(Byte, space)
        space = True
    return result


def StringToHex(string):
    """
    Перевод строки ответа в hex массив
    """
    parsed = [(hex(c)) for c in bytearray(string)]
    return str(parsed)


def DataToHexString(string):
    result = ""
    for CHR in bytearray(string, encoding="utf-8"):
        result = result + HexToStr(CHR)
    return result


def DateTimeToUnix(date_time):
    return int(time.mktime(date_time.timetuple()))


def UnixTimeToDateTime(unix_time):
    return datetime.fromtimestamp(unix_time)


def UnixTimeToString(unix_time):
    return UnixTimeToDateTime(unix_time).strftime("%d.%m.%Y %H:%M:%S")


def string_to_unix_time_bytes(str_date_time):
    """str_date_time строка вида 'YYYY.MM.DD hh:mm:ss'"""
    dt = datetime.strptime(str_date_time, "%Y.%m.%d %H:%M:%S")
    unixtime = DateTimeToUnix(dt)
    return int(unixtime).to_bytes(length=4, byteorder=sys.byteorder)


def NumberToBytes(number, count_of_bytes):
    result = ""
    for i in range(count_of_bytes):
        result = result + HexToStr(number & 0xFF)
        number = number >> 8
    return result


def NumberToArray(number, count_of_bytes):
    result = []
    for i in range(count_of_bytes):
        result.append(number & 0xFF)
        number = number >> 8
    if count_of_bytes == 0:
        while number > 0:
            result.append(number & 0xFF)
            number = number >> 8
    return result


def DateTimeToUnixBytes(date_time):
    return NumberToBytes(DateTimeToUnix(date_time), 4)


def DateTimeToUnixArray(date_time):
    return NumberToArray(DateTimeToUnix(date_time), 4)


def HexToStr(Integer, print_space: bool = True):
    if print_space:
        space = " "
    else:
        space = ""
    return space + ("%X" % Integer).rjust(2, "0")


def ByteStrToList(string):
    array = []
    for byte in string.split(" "):
        array.append(int(byte, 16))
    return array


def StructToList(struct):
    return list(struct.unpack("{}B".format(len(struct)), struct))


def StructToHexString(struct):
    result = ""
    space = False
    for Byte in StructToList(struct):
        result += HexToStr(Byte, space)
        space = True
    return result


def ArrayToHexString(array):
    result = ""
    space = False
    for Byte in array:
        result += HexToStr(Byte, space)
        space = True
    return result


def HexStringToString_InP5(
    hex_string,
):  # декодер из кодировки ККТ- позволяет читать данные в привычной кодировке
    result = ""
    hex_string_arr = hex_string.split(" ")
    for hexSymbol in hex_string_arr:
        b = bytes.fromhex(hexSymbol)
        result += b.decode("cp866")
    return result


def StringToHexString(
    string,
):  # кодировщик в кодировку ККТ (просто вводить аргументы для команды через ';')
    string = str(string)
    result = ""
    for char in string:
        b = bytes(char, "cp866")
        result += bytes.hex(b).upper()
        result += " "
    return result.strip()


def StringSpaceSplit(string, step=2):
    return " ".join([string[i : i + step] for i in range(0, len(string), step)])


def Crc16Table():
    crctable = []
    for i in range(0x100):
        r = (i & 0xFF) << 8
        for j in range(8):
            if r & (1 << 15):
                r = (r << 1) ^ 0x1021
            else:
                r = r << 1
        crctable.append(r & 0xFFFF)
    return crctable


def CRC16_CCITT(p_buf):
    t_ans = 0xFFFF
    table = Crc16Table()
    for BYTE in p_buf:
        t_ans = ((t_ans << 8) & 0xFFFF) ^ table[((t_ans >> 8) & 0xFFFF) ^ BYTE]
    return t_ans


def CRC16_CCITT_XModem(p_buf):
    t_ans = 0x0000
    table = Crc16Table()
    for BYTE in p_buf:
        t_ans = ((t_ans << 8) & 0xFFFF) ^ table[((t_ans >> 8) & 0xFFFF) ^ BYTE]
    return t_ans


# фунцкия вычисляет контрольное число для 10-значного ИНН, требуется 9 символов, строки длиннее обрезаются
def CalcINN10(inn10):
    inn10 = str(inn10)
    if len(inn10) < 9:
        inn10 = inn10.zfill(9)
    a = []
    for i in range(9):
        a.append(int(inn10[i]))
    c = (
        2 * a[0]
        + 4 * a[1]
        + 10 * a[2]
        + 3 * a[3]
        + 5 * a[4]
        + 9 * a[5]
        + 4 * a[6]
        + 6 * a[7]
        + 8 * a[8]
    )
    c = (c % 11) % 10
    return inn10[:9] + str(c)


def CalcINN12(inn12):
    """
    фунцкия вычисляет контрольное число для 12-значного ИНН, требуется 10 символов, строки длиннее обрезаются
    """
    inn12 = str(inn12)
    if len(inn12) < 10:
        inn12 = inn12.zfill(10)
    a = []
    for i in range(10):
        a.append(int(inn12[i]))
    c1 = (
        7 * a[0]
        + 2 * a[1]
        + 4 * a[2]
        + 10 * a[3]
        + 3 * a[4]
        + 5 * a[5]
        + 9 * a[6]
        + 4 * a[7]
        + 6 * a[8]
        + 8 * a[9]
    )
    c1 = (c1 % 11) % 10
    c2 = (
        3 * a[0]
        + 7 * a[1]
        + 2 * a[2]
        + 4 * a[3]
        + 10 * a[4]
        + 3 * a[5]
        + 5 * a[6]
        + 9 * a[7]
        + 4 * a[8]
        + 6 * a[9]
        + 8 * c1
    )
    c2 = (c2 % 11) % 10
    return inn12[:10] + str(c1) + str(c2)


# фунцкия вычисляет ИНН с контрольными числами, длина строки без пробелов определяет ИНН 10 или ИНН 12
def CalcINN(inn):
    inn = str(inn).replace(" ", "")
    if len(inn) == 10:
        return CalcINN10(inn)
    if len(inn) == 12:
        return CalcINN12(inn)


# фунцкия вычисляет РНМ по ИНН и ЗН, требуется 10 символов РНМ, строки длиннее обрезаются
def CalcRNM(full_serial_number, inn12, rnm):
    full_serial_number = str(full_serial_number)[:20]
    inn12 = str(inn12)[:12].strip()
    rnm = str(rnm)[:10]
    if len(rnm) < 10:
        rnm = rnm.zfill(10)
    if len(inn12) < 12:
        inn12 = inn12.zfill(12)
    if len(full_serial_number) < 20:
        full_serial_number = full_serial_number.zfill(20)
    p_buf = []
    for CHAR in bytearray(str(rnm) + inn12 + str(full_serial_number), encoding="utf-8"):
        p_buf.append(CHAR)
    c = CRC16_CCITT(p_buf)
    c = str(c).zfill(6)
    return rnm + c


def GetBit(byte, number):
    return ((1 << number) & byte) >> number


def SetBit(byte, number):
    return byte | (1 << number)


# def SendMsg(Data):
#     try:
#         sock = socket.socket()
#         sock.connect(('localhost', 7567))
#         sock.send(Data)
#         sock.settimeout(30)
#         data = sock.recv(1024)
#         if data == 'OK':
#             print('OK - Succesfully sent socket data ' + Data)
#         else:
#             print('"{}"'.format(data))
#         sock.close()
#         time.sleep(5)
#         del sock
#     except:
#         print('BAD (time out!) - not sent socket data ' + Data)


def SendMsg(data):
    try:
        file = open(r"C:\ProgramData\ATOL\OT\emr.sok", "a", encoding="utf-8")
        file.write(data + "\n")
        file.close()
    except Exception as e:
        print(e)


def End(code, driver=None):
    if driver is not None and not driver.gAutoTest:
        return
    try:
        time.sleep(2)
        SendMsg("END " + code)
        time.sleep(2)
        exit()
    except Exception as e:
        print("Не удалось отправить данные скрипта", e)
        pass


def valid_IP(ip_adress):
    octets = str(ip_adress).split(".")
    if len(octets) != 4:
        return False
    for octet in octets:
        # noinspection PyBroadException
        try:
            if int(octet) > 255:
                return False
        except Exception:
            return False
    return True


def valid_MAC(mac):
    octets = str(mac).split(":")
    if len(octets) != 6:
        octets = str(mac).split("-")
    if len(octets) != 6:
        return False
    for octet in octets:
        # noinspection PyBroadException
        try:
            if int(octet, 16) > 255:
                return False
        except Exception:
            return False
    return True


# noinspection PyPep8Naming
class Hex:
    # StrHex:           '404132'
    # StrHexSpace:      '40 41 32'
    # List:             [0x40, 0x41, 0x32]
    # Bytes (Symbols):  '@A2'

    Array = []

    def clear(self):
        self.Array = []

    def setHexData(self, data, _bytes=None):
        self.Array = self.__HexData__(data, _bytes)

    def addHexData(self, data, _bytes=None):
        self.Array += self.__HexData__(data, _bytes)

    def setSymbolsData(self, data, _bytes=None):
        self.Array = self.__HexData__(data, _bytes=True)

    def addSymbolsData(self, data, _bytes=None):
        self.Array += self.__HexData__(data, _bytes=True)

    def setIntData(self, data, LE=True, BytesCount=None):
        self.Array = self.__IntData__(data, LE, BytesCount)

    def addIntData(self, data, LE=True, BytesCount=None):
        self.Array += self.__IntData__(data, LE, BytesCount)

    def __IntData__(self, data, LE=True, BytesCount=None):
        Result = []
        if BytesCount is None:
            while data > 0:
                Result.append(data & 0xFF)
                data = data >> 8
        else:
            for i in range(BytesCount):
                Result.append(data & 0xFF)
                data = data >> 8
        if not LE:
            Result.reverse()
        return Result

    def __HexData__(self, data, _bytes):
        Result = []
        if type(data) == list:
            Result = data
        elif type(data) == str:
            if not _bytes:
                if re.match("^[A-Fa-f0-9]*$", data) is not None:
                    Index = 0
                    while Index < len(data):
                        Result.append(int(data[Index : Index + 2], 16))
                        Index += 2
                    return Result
                elif re.match("^[A-Fa-f0-9 ]*$", data) is not None:
                    for byte in data.split(" "):
                        if len(byte) > 2:
                            Result = []
                            return Result
                            # Result.append(int(byte, 16))
                    return Result
            for symbol in bytearray(data, encoding="utf-8"):
                Result.append(symbol)
        return Result

    def toList(self):
        return self.Array

    def toBytes(self):
        Result = ""
        for elem in self.Array:
            Result += chr(elem)
        return Result

    def toSymbols(self):
        return self.toBytes()

    def toStrHex(self):
        Result = ""
        for elem in self.Array:
            Result += "{Hex:02X}".format(Hex=elem)
        return Result

    def toStrHexSpace(self):
        Result = ""
        for elem in self.Array:
            Result += "{Hex:02X} ".format(Hex=elem)
        return Result.strip()

    def __str__(self):
        return self.toStrHexSpace()


if __name__ == "__main__":
    h = Hex()
    h.setIntData(0)
    # print(StringToHexString(''))



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="ofdcomparer",
    version="1.0.6",
    long_description=long_description,
    description="Библиотека для сравнивания ФД из ФН и ОФД",
    packages=["ofdcomparer"],
    author_email="k.kabisova@atol.ru",
    install_requires=[
        "requests==2.31.0",
        "requests-toolbelt==1.0.0",
        "urllib3==2.0.6",
        "zipp==3.17.0",
    ],
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



import json
import logging
import threading
import time

from ofdcomparer.dto_error_descriptions import get_dto_error_code_description
from ofdcomparer.libfptr10 import IFptr


class DTO10Helper:
    def __init__(self, fptr=None):
        self.fptr = fptr
        if fptr is None:
            while True:
                try:
                    self.fptr = IFptr()
                except Exception:
                    raise Exception("Error creating DTO 10")
                if self.fptr.interface.value is not None:
                    break

        self.dto10_version = self.fptr.version().decode("cp866")

        self.ErrorCode = None
        self.ErrorDescription = None
        self.JsonAnswer = None

        if fptr is None:
            self.fptr.setSingleSetting(
                self.fptr.LIBFPTR_SETTING_MODEL, str(self.fptr.LIBFPTR_MODEL_ATOL_AUTO)
            )
        self.RNM = self.get_rnm_number()
        self.FN = self.get_fn_number()

    def get_information_and_status(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_STATUS
        )
        self.fptr.queryData()
        self.request_error_code()
        serial_number = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SERIAL_NUMBER)
        return serial_number

    def open(self):
        self.fptr.open()
        return self.request_error_code()

    def close(self):
        self.fptr.close()
        return self.request_error_code()

    def request_error_code(self, flag_all_kkt_settings=False):
        self.ErrorCode = self.fptr.errorCode()
        self.ErrorDescription = self.fptr.errorDescription()
        pytest_dto_error_description = get_dto_error_code_description(self.ErrorCode)
        if self.ErrorDescription != pytest_dto_error_description:
            logging.error(f"Код ошибки ДТО: {self.ErrorCode}")
            logging.error(
                f"ДТО [{self.ErrorDescription}] <-> "
                f"[{pytest_dto_error_description}] const.dto_error_descriptions"
            )
            logging.error(
                "НЕОБХОДИМО АКТУАЛИЗИРОВАТЬ СЛОВАРЬ ALL_DTO10_ERROR_DESCRIPTIONS"
            )
        if self.ErrorCode != self.fptr.LIBFPTR_OK:
            if (
                    flag_all_kkt_settings
                    and self.ErrorCode == self.fptr.LIBFPTR_ERROR_NOT_SUPPORTED
            ):
                pass
            else:
                logging.error(
                    "[ДТО]: Код ошибки: {} [{}]".format(
                        self.ErrorCode, self.ErrorDescription
                    )
                )
        return self.ErrorCode

    def get_last_fiscal_document_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_FN_DATA_TYPE, self.fptr.LIBFPTR_FNDT_LAST_DOCUMENT
        )
        self.fptr.fnQueryData()
        self.request_error_code()
        return int(self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_DOCUMENT_NUMBER))

    def get_last_fd_number(self):
        logging.debug("get_last_fd_number()")
        try:
            json_task = {"type": "getFnStatus"}
            fn_status = self.execute_json(json_task)
            logging.debug("СТАТУС ФН: %s", fn_status)
            last_fd_number = fn_status["fnStatus"]["fiscalDocumentNumber"]
            logging.debug(f"get_last_fd_number() > {last_fd_number}")
            logging.info("ФД: %s", last_fd_number)
            return last_fd_number
        except Exception as e:
            logging.error("get_last_fd_number error: {}".format(e))
            logging.debug("get_last_fd_number() > None")
            return None

    def get_fd_from_fn(self, fd: int):
        """
        Извлечение ФД из ФН
        """
        logging.debug(f"get_fd_from_fn() < {fd}")
        if fd is None:
            logging.debug(f"get_fd_from_fn() > None")
            return None
        # JSON задание для чтения документа из ФН

        json_task = {
            "type": "getFnDocument",
            "fiscalDocumentNumber": int(fd),
            "withRawData": False,
        }
        fd_fn = self.execute_json(json_task)
        if fd_fn is None:
            return None
        if isinstance(fd_fn, dict):
            fd_fn = fd_fn["documentTLV"]
            return fd_fn
        return None

    def connect_to_kkt_by_usb(self, usb_device_path="auto"):
        logging.debug("Устанавливаем соединение с ККТ по USB...")
        self.set_connection_settings_to_usb(usb_device_path)
        if self.open() != self.fptr.LIBFPTR_OK:
            logging.error("Не удалось установить соединение с ККТ")
            return False
        if not self.is_connection_type_usb():
            return False
        logging.info("Соединение с ККТ установлено по USB")
        return True

    def connect_to_kkt_by_ethernet(self, ip="127.0.0.1", port="5555"):
        logging.debug(f"Устанавливаем соединение по ip {ip}:{port}")
        self.set_connection_settings_to_ethernet(ip, port)
        err = self.open()
        if err != self.fptr.LIBFPTR_OK:
            logging.info(f"Ошибка - не удалось установить соединение с ККТ ({err})")
            return False
        if not self.is_connection_type_ethernet():
            return False
        logging.info(
            f"Соединение с ККТ установлено по Ethernet. ip=[{ip}] port=[{port}]"
        )
        return True

    def set_connection_settings_to_ethernet(self, ip, port):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_TCPIP)
        )
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_IPADDRESS, str(ip))
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_IPPORT, str(port))
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_ethernet(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_TCPIP:
            logging.error(
                f"Способ связи с ККТ [{connection_type}], а должен быть [Ethernet]"
            )
            return False
        return True

    def set_connection_settings_to_usb(self, usb_device_path):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_USB)
        )
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_USB_DEVICE_PATH, str(usb_device_path)
        )
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_usb(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        self.request_error_code()
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_USB:
            logging.error(
                "Способ связи с ККТ [{}], а должен быть [USB]".format(connection_type)
            )
            return False
        return True

    def connect_to_kkt_by_rs(self, com="COM1", baudrate=115200):
        print("Устанавливаем соединение по RS")
        if self.set_connection_settings_to_rs(com, baudrate) != self.fptr.LIBFPTR_OK:
            print("Не удалось установить настройки драйвера для подключения к ККТ")
            return False
        if self.open() != self.fptr.LIBFPTR_OK:
            print("Не удалось установить соединение с ККТ")
            return False
        if not self.is_connection_type_rs():
            return False
        print(
            "Соединение с ККТ установлено по RS. com=[{}] baudrate=[{}]".format(
                com, baudrate
            )
        )
        return True

    def set_connection_settings_to_rs(self, com, baudrate):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_MODEL, str(self.fptr.LIBFPTR_MODEL_ATOL_AUTO)
        )
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_COM)
        )
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_COM_FILE, com)
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_BAUDRATE, str(baudrate))
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_rs(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        self.request_error_code()
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_COM:
            logging.error(
                "Способ связи с ККТ [{}], а должен быть [RS]".format(connection_type)
            )
            return False
        return True

    def get_model_name(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_MODEL_INFO
        )
        self.fptr.queryData()
        model_name = str(self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_MODEL_NAME))
        return model_name

    def set_no_print_flag(self, no_print):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_REPORT_ELECTRONICALLY, no_print)
        return True

    def reset_settings(self):
        self.fptr.resetSettings()
        return self.request_error_code()

    def get_fatal_errors_status(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_FATAL_STATUS
        )
        self.fptr.queryData()

        self.request_error_code()

        no_serial_number = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_NO_SERIAL_NUMBER
        )
        rtc_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_RTC_FAULT)
        settings_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_SETTINGS_FAULT)
        counter_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_COUNTERS_FAULT)
        user_memory_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_USER_MEMORY_FAULT
        )
        service_counters_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_SERVICE_COUNTERS_FAULT
        )
        attributes_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_ATTRIBUTES_FAULT
        )
        fn_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_FN_FAULT)
        invalid_fn = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_INVALID_FN)
        hard_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_HARD_FAULT)
        memory_manager_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_MEMORY_MANAGER_FAULT
        )
        script_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_SCRIPTS_FAULT)
        wait_for_reboot = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_WAIT_FOR_REBOOT
        )
        universal_counters_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_UNIVERSAL_COUNTERS_FAULT
        )
        commodities_table_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_COMMODITIES_TABLE_FAULT
        )

        return (
            no_serial_number,
            rtc_fault,
            settings_fault,
            counter_fault,
            user_memory_fault,
            service_counters_fault,
            attributes_fault,
            fn_fault,
            invalid_fn,
            hard_fault,
            memory_manager_fault,
            script_fault,
            wait_for_reboot,
            universal_counters_fault,
            commodities_table_fault,
        )

    def reboot_kkt(self):
        self.fptr.deviceReboot()
        return self.request_error_code()

    def set_setting_id(self, _id, _value):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_ID, _id)
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_VALUE, _value)
        self.fptr.writeDeviceSetting()
        self.request_error_code()
        if self.ErrorCode != self.fptr.LIBFPTR_OK:
            return self.ErrorCode
        self.fptr.commitSettings()
        return self.request_error_code()

    def get_setting_id(self, _id):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_ID, _id)
        self.fptr.readDeviceSetting()
        self.request_error_code()
        return self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SETTING_VALUE)

    def print_string(
            self,
            text=None,
            alignment=None,
            wrap=None,
            font=None,
            double_width=None,
            double_height=None,
            linespacing=None,
            brightness=None,
            store_in_journal=None,
    ):
        self.fptr.beginNonfiscalDocument()
        if text is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_TEXT, text)
        if alignment is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_ALIGNMENT, alignment)
        if wrap is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_TEXT_WRAP, wrap)
        if font is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT, font)
        if double_width is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_WIDTH, double_width)
        if double_height is not None:
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT, double_height
            )
        if linespacing is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_LINESPACING, linespacing)
        if brightness is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_BRIGHTNESS, brightness)
        if store_in_journal is not None:
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_STORE_IN_JOURNAL, store_in_journal
            )
        self.fptr.printText()
        self.fptr.endNonfiscalDocument()
        return self.request_error_code()

    def print_custom_text(self, text_tuple: tuple):
        self.fptr.open()
        self.fptr.beginNonfiscalDocument()

        for text in text_tuple:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_TEXT, text)
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_ALIGNMENT, self.fptr.LIBFPTR_ALIGNMENT_CENTER
            )
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_TEXT_WRAP, self.fptr.LIBFPTR_TW_WORDS
            )
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT, 0)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT, False)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_WIDTH, False)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FORMAT_TEXT, False)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_LINESPACING, 0)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_BRIGHTNESS, 0)
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_STORE_IN_JOURNAL, True)
            self.fptr.printText()
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_PRINT_FOOTER, False)
        self.fptr.endNonfiscalDocument()

    # def set_no_print_flag(self, no_print):
    #     self.fptr.setParam(self.fptr.LIBFPTR_PARAM_REPORT_ELECTRONICALLY, no_print)

    def get_fn_info(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_FN_DATA_TYPE, self.fptr.LIBFPTR_FNDT_FN_INFO
        )
        self.fptr.fnQueryData()
        self.request_error_code()

        serial = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SERIAL_NUMBER)
        version = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_FN_VERSION)
        type = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_TYPE)
        state = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_STATE)
        flags = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_FLAGS)

        need_replacement = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_NEED_REPLACEMENT
        )
        exhausted = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_RESOURCE_EXHAUSTED
        )
        memory_overflow = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_MEMORY_OVERFLOW
        )
        ofd_timeout = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_FN_OFD_TIMEOUT)
        critical_error = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_CRITICAL_ERROR
        )

        return (
            serial,
            version,
            type,
            state,
            flags,
            need_replacement,
            exhausted,
            memory_overflow,
            ofd_timeout,
            critical_error,
        )

    def init_mgm(self):
        self.fptr.initMgm()
        return self.request_error_code()

    def get_last_doc_with_template(self, tempalate, variable_settings=None):
        last_fn_doc_number = self.get_last_fiscal_document_number()
        last_fn_doc_str = None
        for _ in range(3):
            request_last_fn_doc = (
                    '{"type": "getFnDocument", "fiscalDocumentNumber": %i}'
                    % last_fn_doc_number
            )
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, request_last_fn_doc)
            self.fptr.processJson()
            last_fn_doc_str = self.fptr.getParamString(
                self.fptr.LIBFPTR_PARAM_JSON_DATA
            )
            if last_fn_doc_str != "":
                break
        last_fn_doc_dic = json.loads(str(last_fn_doc_str))
        last_fn_doc_dic = last_fn_doc_dic["documentTLV"]
        logging.info(
            "\nФД %s, JSON из ФН: %s"
            % (
                str(last_fn_doc_number),
                json.dumps(
                    last_fn_doc_dic, sort_keys=True, indent=4, ensure_ascii=False
                ),
            )
        )

        return last_fn_doc_dic

    def execute_json(self, json_task, timeout=30):
        logging.debug(f"execute_json() < json_task {json_task}, timeout {timeout}")
        try:
            self.fptr.open()
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, json.dumps(json_task))
            self.fptr.processJson()
            response_raw = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_JSON_DATA)
            if response_raw is not None:
                try:
                    response = json.loads(response_raw)
                    logging.debug(f"execute_json() > {response}")
                    return response
                except json.decoder.JSONDecodeError as e:
                    logging.error(
                        "[ERROR] error with json.loads from response: {}".format(e)
                    )
        except ValueError as e:
            logging.error("Exception in execute_json(): {}".format(e))
        finally:
            self.fptr.close()  # Закрытие ДТО
        logging.debug(f"execute_json() > None")
        return None

    def get_last_fd_number(self):
        logging.debug("get_last_fd_number()")
        try:
            json_task = {"type": "getFnStatus"}
            fn_status = self.execute_json(json_task)
            logging.debug("СТАТУС ФН: %s", fn_status)
            last_fd_number = fn_status["fnStatus"]["fiscalDocumentNumber"]
            logging.debug(f"get_last_fd_number() > {last_fd_number}")
            logging.info("ФД: %s", last_fd_number)
            return last_fd_number
        except Exception as e:
            logging.error("get_last_fd_number error: {}".format(e))
            logging.debug("get_last_fd_number() > None")
            return None

    def get_fd_from_fn(self, fd: int):
        """
        Извлечение ФД из ФН
        """
        logging.debug(f"get_fd_from_fn() < {fd}")
        if fd is None:
            logging.debug(f"get_fd_from_fn() > None")
            return None
        # JSON задание для чтения документа из ФН
        json_task = {
            "type": "getFnDocument",
            "fiscalDocumentNumber": int(fd),
            "withRawData": False,
        }
        fd_fn = self.execute_json(json_task)
        if fd_fn is None:
            return None
        if isinstance(fd_fn, dict):
            fd_fn = fd_fn["documentTLV"]
            # formatted_fd_fn = json.dumps(fd_fn, sort_keys=True, indent=4, ensure_ascii=False)
            return fd_fn
        return None

    def get_fn_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_CACHE_REQUISITES
        )
        self.fptr.queryData()

        serialNumber = self.fptr.getParamString(
            self.fptr.LIBFPTR_PARAM_FN_SERIAL_NUMBER
        )
        ecrRegNumber = self.fptr.getParamString(
            self.fptr.LIBFPTR_PARAM_ECR_REGISTRATION_NUMBER
        )
        ofdVatin = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_OFD_VATIN)
        fnsUrl = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_FNS_URL)
        ffdVersion = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FFD_VERSION)
        return serialNumber

    def get_rnm_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_CACHE_REQUISITES
        )
        self.fptr.queryData()

        ecrRegNumber = self.fptr.getParamString(
            self.fptr.LIBFPTR_PARAM_ECR_REGISTRATION_NUMBER
        )

        return ecrRegNumber

    def get_not_sent_fd_qty(
            self,
    ):
        logging.debug(f"get_not_sent_fd_qty()")
        task = {"type": "ofdExchangeStatus"}
        ofdExchangeStatus = self.execute_json(task)
        if ofdExchangeStatus is not None:
            not_sent_fd_qty = ofdExchangeStatus["status"]["notSentCount"]
            logging.debug(f"get_not_sent_fd_qty() %s", not_sent_fd_qty)
            return not_sent_fd_qty
        logging.debug(f"get_not_sent_fd_qty() None")
        return None

    def wait_for_sent_all_fd(self, timeout: int = 600):
        logging.debug(f"wait_for_sent_all_fd() timeout %s", timeout)
        start_time = time.time()
        while self.get_not_sent_fd_qty() != 0:
            if time.time() - start_time > timeout:
                logging.debug(f"get_not_sent_fd_qty() False")
                return False
            time.sleep(3)
        logging.debug(f"get_not_sent_fd_qty() True")
        return True

    def process_json(self, json_task, answer_indent=4, flag_all_kkt_settings=False):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, json_task)
        self.fptr.processJson()
        error = self.request_error_code(flag_all_kkt_settings=flag_all_kkt_settings)
        json_str_answer = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_JSON_DATA)
        if json_str_answer != "":
            self.JsonAnswer = None
            try:
                self.JsonAnswer = json.dumps(
                    json.loads(json_str_answer),
                    ensure_ascii=False,
                    indent=answer_indent,
                )
            except ValueError:
                logging.error("Не удалось считать структуру из json-ответа")
        return error

    def execute_mass_json(self, json_task, quantity, timeout=30):
        logging.debug(f"execute_json() < json_task {json_task}, timeout {timeout}")
        print(f"execute_json() < json_task {json_task}, timeout {timeout}")
        try:
            if not self.is_connected():
                if not self.wait_to_connect(timeout=timeout):
                    return False
            self.fptr.open()
            count = 0
            while count < quantity:
                self.fptr.setParam(
                    self.fptr.LIBFPTR_PARAM_JSON_DATA, json.dumps(json_task)
                )
                self.fptr.processJson()
                print(count)
                count += 1
        except:
            logging.error("Exception in execute_json():")
        finally:
            self.fptr.close()  # Закрытие ДТО
        logging.debug(f"execute_json() > None")
        return None

    def is_connected(self, timeout=5):
        logging.debug("ККТ is_connected() timeout %s", timeout)
        connected = False

        def check_connection():
            nonlocal connected
            self.fptr.open()
            if self.fptr.isOpened():
                self.fptr.close()
                logging.debug("ККТ is_connected() True")
                connected = True
            self.fptr.close()

        timer = threading.Timer(timeout, check_connection)
        timer.start()

        while not connected and timer.is_alive():
            time.sleep(0.1)

        timer.cancel()

        logging.debug("ККТ is_connected() %s", connected)
        return connected

    def wait_to_connect(self, timeout=300):
        logging.debug(f"wait_to_connect() timeout={timeout}")
        start_time = time.time()
        while not self.is_connected() and not time.time() - start_time > timeout:
            logging.debug(
                f"Не получен ответ от ККТ, time {time.time() - start_time} timeout {timeout}"
            )
        if self.is_connected():
            logging.debug("ККТ доступна")
            return True
        logging.debug("ККТ не отвечает")
        return False
